local GridFrame = Grid:GetModule("GridFrame")

GridTooltip = GridFrame:NewModule("GridTooltip")

GridTooltip.defaultDB = {
}

local lastMouseOverFrame

function GridTooltip.SetIndicator(frame, indicator, color, text, value, maxValue, texture, start, duration, stack)
	
	if texture and indicator == "icon" then
        if frame.unit and UnitExists(frame.unit)then
            frame.ToolTip = nil
            local i = 1
            local name, rank, icon, count, buffType, duration, expirationTime, source, isStealable, _, id = UnitDebuff(frame.unit, i)
            while name do 
                if icon == texture then
                    frame.ToolTip = i
                    if lastMouseOverFrame then
                        GridTooltip.OnEnter(lastMouseOverFrame)
                    end
                    break
                end
                i = i + 1
                name, rank, icon, count, buffType, duration, expirationTime, source, isStealable, _, id  = UnitDebuff(frame.unit, i)
            end
        end
	end
end

function GridTooltip.ClearIndicator(frame, indicator)   
    if indicator == "icon" then
        frame.ToolTip = nil
    end
end

function GridTooltip.CreateFrames(gridFrameObj, frame)
    local f = frame
    frame:HookScript("OnEnter", GridTooltip.OnEnter)
	frame:HookScript("OnLeave", GridTooltip.OnLeave)
    --frame.IconBG.frame = frame --oh god!
end

local function EnumerateTooltipLines_helper(...)
    local res = ""
    for i = 1, select("#", ...) do
        local region = select(i, ...)
        if region and region:GetObjectType() == "FontString" then
            local text = region:GetText() -- string or nil
            if text then
                res = res..text.."\n"
            end
        end
    end
    return res
end

function EnumerateTooltipLines(tooltip) -- good for script handlers that pass the tooltip as the first argument.
    return EnumerateTooltipLines_helper(tooltip:GetRegions())
end

function GridTooltip.OnEnter(iconFrame)
    local frame = iconFrame--.frame
    local unitid = frame.unit
    if not unitid then return end
    lastMouseOverFrame = iconFrame
    
    if not frame.ToolTip then return end 
    GameTooltip_SetDefaultAnchor(GameTooltip, UIParent)
    if type(frame.ToolTip) == "string" then
        GameTooltip:SetText( frame.ToolTip )
    elseif frame.ToolTip then
        GameTooltip:SetUnitDebuff(unitid, frame.ToolTip)
        frame.ToolTip = "Cached:\n"..EnumerateTooltipLines(GameTooltip)
    end
    GameTooltip:Show()
end

function GridTooltip.OnLeave(iconFrame)
    GameTooltip:Hide()
    if lastMouseOverFrame == iconFrame then
        lastMouseOverFrame = nil
    end
end

function GridTooltip:OnInitialize()
   if not self.db then
		self.db = Grid.db:RegisterNamespace(self.moduleName, { profile = self.defaultDB or { } })
	end
    
    --hook the hell out of GridFrame :-)
    hooksecurefunc(GridFrame, "InitializeFrame", GridTooltip.CreateFrames)
    hooksecurefunc(GridFrame.prototype, "SetIndicator", GridTooltip.SetIndicator)
    hooksecurefunc(GridFrame.prototype, "ClearIndicator", GridTooltip.ClearIndicator)
end

function GridTooltip:OnEnable()
end

function GridTooltip:OnDisable()
end

function GridTooltip:Reset(frame)
end